/* Open Digita Services  --  Camera Device Protocol definitions.

   Copyright (C) 1998 James C. Thompson

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc., 59
   Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/*--------------------------------------------------------------------------
 * System include files
 */ 

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/stat.h>
#include <time.h>
#include <utime.h>

/*--------------------------------------------------------------------------
 * Local include files
 */ 

#include "ods.h"

/* Start Editing Here: */

extern void dump_ptvs(PNameTypeValueStruct *);

void tolowers(char *s)
{
  for (; *s; s++)
    {
      if (isalpha(*s))
	{
	  *s = tolower(*s);
	}
    }
}

void set_file_utimes(ODSCamera camera, ResFileItem *file)
{
  time_t caltime;
  struct utimbuf timbuf;

  UInteger reslength;
  PNameTypeValueStruct values[100];

  unsigned long ptvsdate;
  unsigned long ptvstime;

  struct tm vtime;
    
  DOSName localfile;

  memcpy(&localfile, &(file->DOSName), sizeof(DOSName));
  tolowers(localfile);

  odsGetFileTag(camera, (const FileNameStruct *) file, 'date', &reslength,
		values, 1);

  ptvsdate = values[0].Value.UInt;
    
  odsGetFileTag(camera, (const FileNameStruct *) file, 'time', &reslength,
		values, 1);

  ptvstime = values[0].Value.UInt;

  vtime.tm_year = ((ptvsdate >> 4) & 0x0f) * 10 + (ptvsdate & 0x0f);
  vtime.tm_mon =
    ((ptvsdate >> 20) & 0x0f) * 10 + ((ptvsdate >> 16) & 0x0f) - 1;
  vtime.tm_mday = 
    ((ptvsdate >> 12) & 0x0f) * 10 + ((ptvsdate >> 8) & 0x0f) - 1;
  vtime.tm_hour = ((ptvstime >> 20) & 0x0f) * 10 + ((ptvstime >> 16) & 0x0f);
  vtime.tm_min = ((ptvstime >> 12) & 0x0f) * 10 + ((ptvstime >> 8) & 0x0f);
  vtime.tm_sec = ((ptvstime >> 4) & 0x0f) * 10 + (ptvstime & 0x0f);

  caltime = mktime(&vtime);

  timbuf.actime = caltime;
  timbuf.modtime = caltime;

  utime(localfile, &timbuf);
}

void test_get_file_data(ODSCamera camera, ResFileItem *file)
{
  PartialTag tag = { 0, 0, 0 };
  char buffer[20000];
  int fd;
  struct stat st;

  unsigned int starting_offset = 0;
  unsigned int target_length = 20000;
  unsigned int length = 0;
  unsigned int request_length = 20000;

  DOSName localfile;
  memcpy(&localfile, &(file->DOSName), sizeof(DOSName));
  tolowers(localfile);
  
  if (file->FileLength < 0)
    {
      printf("%s: still processing, skipping.\n", localfile);
      return;
    }

  /* If the file exists, and has the same length, assume it's already
     been downloaded, and skip it.  This check could stand to be more
     thorough: access permission, type, etc. */

  if (stat(localfile, &st) == 0)
    {
      if (st.st_size == file->FileLength)
	{
	  printf("%s: already downloaded, skipping.\n", localfile);
	  return;
	}
      else
	{
	  starting_offset = st.st_size;
	  printf("%s: continuing from offset %d.\n", localfile, 
		 starting_offset);
	}
    }

  printf("%s: size = %ld, flags = %08lx\n",
	 localfile, file->FileLength, file->FileStatus);

  fd = open(localfile, O_RDWR|O_CREAT|O_APPEND, 0666);

  tag.Offset = starting_offset;
  tag.Length = request_length;

  while (length < target_length)
    {
      odsGetFileData(camera, (FileNameStruct *) file, 0, &tag, buffer);

      if (tag.Length < request_length)
	{
	  request_length = tag.Length;
	}

      write(fd, buffer, request_length);

      length += request_length;
      tag.Offset += request_length;

      printf("%s: %ld bytes of %ld.\n", localfile, tag.Offset, tag.Filesize);

      request_length = tag.Length;
      target_length = tag.Filesize - starting_offset;
    }

  close(fd);

  /* Now that the file has been downloaded, set the access and mod
     times in it to match the date and time the photo was taken. */

  set_file_utimes(camera, file);
}

void test_get_file_tag(ODSCamera camera,  ResFileItem *file)
{
  int i;
  UInteger reslength;
  PNameTypeValueStruct values[100];

  printf("**************** %s\n", file->DOSName);
  odsGetFileTag(camera, (const FileNameStruct *) file, 0, &reslength,
		values, 100);

  for (i = 0; i < reslength; i++)
    {
      dump_ptvs(values + i);
    }

  printf("\n");
}

void test_get_file_list(ODSCamera camera)
{
  int i;
  UInteger reslength;
  ResFileItem files[100];
  
  odsGetFileList(camera, 1, 0, &reslength, files, 100);

  printf("%ld file%s in camera...\n", reslength, 
	 (reslength == 1) ? "" : "s");
  for (i = 0; i < reslength; i++)
    {
/*       tolowers(files[i].PathName); */
/*       tolowers(files[i].DOSName); */

      test_get_file_data(camera, files + i);
/*       test_get_file_tag(camera, files + i); */
    }
}

int main(int argc, const char *argv[])
{
  ODSCamera camera;
/*   ResLength reslength; */
/*   PNameTypeValueStruct values[100]; */
/*   int i; */
  UInteger powerstate;
/*   FileNameStruct name = { 2, "DC260_01/", "P0000741.JPG" }; */

  camera = ODSInitialize(&argc, argv);

#if 0
  odsGetProductInfo(camera, 0, &reslength, values, 100);
  
  for (i = 0; i < reslength; i++)
    {
      dump_ptvs(values + i);
    }
#endif

  odsGetPowerMode(camera, &powerstate);

  test_get_file_list(camera);

  ODSFinalize(camera);

  return 0;
}
