/*--------------------------------------------------------------------------
   Open Digita Services  --  Camera Device Protocol definitions.

   Copyright (C) 1998 James C. Thompson

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc., 59
   Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/*--------------------------------------------------------------------------
 * System include files
 */ 

#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>

/*--------------------------------------------------------------------------
 * Local include files
 */ 

#include "defs.h"
#include "format-mem.h"

unsigned int checksum(const unsigned char *buffer, int len)
{
  int i;
  int limit = len - 1;
  unsigned int sum = 0;

  for (i = 0; i < limit; i++)
    {
      sum += *(buffer++);
    }

  return sum & 0xff;
}

int verify_checksum(const unsigned char *buffer, int len)
{
  unsigned int sum = checksum(buffer, len);

  if (sum == buffer[len - 1])
    {
#if defined(DEBUG)
      printf("checksums match\n");
#endif
      return 1;
    }
  else
    {
#if defined(DEBUG)
      printf("checksums differ: received = %02x, computed = %02x\n",
	     *buffer, sum);
#endif
      return 0;
    }
}

void set_checksum(unsigned char *buffer, int len)
{
  unsigned int sum = checksum(buffer, len);

  buffer[len - 1] = sum;
}

int read_fully(int fd, unsigned char *buffer, int len)
{
#ifdef DEBUG
  unsigned char *buforig = buffer;
#endif

  int bytes_read = 0;
  int bytes_remaining = len;

  while (bytes_read < len)
    {
      int read_here = read(fd, buffer, bytes_remaining);
      if (read_here > 0)
	{
	  bytes_remaining -= read_here;
	  bytes_read += read_here;
	  buffer += read_here;
	}
#if defined(DEBUG)
      else
	{
	  printf("doh!");
	}
#endif
    }

#ifdef DEBUG
  printf("read:\n");
  format_chars(stdout, buforig, 0, len);
#endif

  return bytes_read;
}

int write_fully(int fd, const unsigned char *buffer, int len)
{
  int bytes_written = 0;
  int bytes_remaining = len;

#ifdef DEBUG
  printf("writing...\n");
  format_chars(stdout, buffer, 0, len);
#endif

  while (bytes_written < len)
    {
      int written_here = write(fd, buffer, bytes_remaining);
      if (written_here > 0)
	{
	  bytes_remaining -= written_here;
	  bytes_written += written_here;
	  buffer += written_here;
	}
#if defined(DEBUG)
      else
	{
	  printf("doh!");
	}
#endif
    }

  return bytes_written;
}

#if defined(DEBUG)
void dump(const char *message, int c, const char *buffer)
{
  printf("%s: %d bytes:\n", message, c);
  format_chars(stdout, buffer, 0, c);
  printf("\n");
}
#endif
